import React, { lazy, Suspense, useRef, RefObject } from "react";
import { Row, Col, Spin, Empty, Button } from "antd";
import {
  BarChartOutlined,
  PieChartOutlined,
  DownloadOutlined,
} from "@ant-design/icons";
import DashboardCard from "../../../components/presentation/dashboard-card";
import { message } from "antd";
import { EChartsOption } from "echarts-for-react";

// 懒加载图表组件
const ReactECharts = lazy(() => import("echarts-for-react"));

interface DeptData {
  id: string;
  name: string;
  count: number;
}

interface DepartmentChartsProps {
  chartView: string;
  filteredDeptData: DeptData[];
  pieOptions: EChartsOption;
  barOptions: EChartsOption;
  handleDeptSelection: (params: any) => void;
  handleExportChart: (chartType: string, chartRef: RefObject<any>) => void;
}

export default function DepartmentCharts({
  chartView,
  filteredDeptData,
  pieOptions,
  barOptions,
  handleDeptSelection,
  handleExportChart,
}: DepartmentChartsProps): React.ReactElement {
  const pieChartRef = useRef(null);
  const barChartRef = useRef(null);

  return (
    <Row gutter={[16, 16]}>
      {/* 条形图 */}
      {(chartView === "all" || chartView === "bar") && (
        <Col
          xs={24}
          sm={24}
          md={chartView === "all" ? 12 : 24}
          lg={chartView === "all" ? 12 : 24}
          xl={chartView === "all" ? 12 : 24}
        >
          <DashboardCard
            title={
              <div className="flex items-center justify-between w-full">
                <div>
                  <BarChartOutlined className="mr-2" />
                  <span>部门人员分布 - 条形图</span>
                </div>
                <Button
                  type="text"
                  icon={<DownloadOutlined />}
                  onClick={() => handleExportChart("bar", barChartRef)}
                  size="small"
                >
                  导出
                </Button>
              </div>
            }
            className="min-h-96"
            contentClassName="flex items-center justify-center"
          >
            {filteredDeptData.length > 0 ? (
              <Suspense fallback={<Spin />}>
                <ReactECharts
                  ref={barChartRef}
                  option={barOptions}
                  style={{ height: "360px", width: "100%" }}
                  notMerge={true}
                  lazyUpdate={true}
                />
              </Suspense>
            ) : (
              <Empty description="暂无部门数据" className="my-8" />
            )}
          </DashboardCard>
        </Col>
      )}

      {/* 饼图 */}
      {(chartView === "all" || chartView === "pie") && (
        <Col
          xs={24}
          sm={24}
          md={chartView === "all" ? 12 : 24}
          lg={chartView === "all" ? 12 : 24}
          xl={chartView === "all" ? 12 : 24}
        >
          <DashboardCard
            title={
              <div className="flex items-center justify-between w-full">
                <div>
                  <PieChartOutlined className="mr-2" />
                  <span>部门人员分布 - 饼图</span>
                </div>
                <Button
                  type="text"
                  icon={<DownloadOutlined />}
                  onClick={() => handleExportChart("pie", pieChartRef)}
                  size="small"
                >
                  导出
                </Button>
              </div>
            }
            className="min-h-96"
            contentClassName="flex items-center justify-center"
          >
            {filteredDeptData.length > 0 ? (
              <Suspense fallback={<Spin />}>
                <ReactECharts
                  ref={pieChartRef}
                  option={pieOptions}
                  style={{ height: "360px", width: "100%" }}
                  notMerge={true}
                  lazyUpdate={true}
                  onEvents={{
                    click: handleDeptSelection,
                  }}
                />
              </Suspense>
            ) : (
              <Empty description="暂无部门数据" className="my-8" />
            )}
          </DashboardCard>
        </Col>
      )}
    </Row>
  );
}
