import React, { useEffect, useState, useMemo, RefObject } from "react";
import { Spin, message } from "antd";
import { api } from "@nice/client";

import StatisticCards from "./StatisticCards";
import ChartControls from "./ChartControls";
import DepartmentCharts from "./DepartmentCharts";
import DepartmentTable from "./DepartmentTable";
import { getPieChartOptions, getBarChartOptions } from "./char-options";
interface DeptData {
  id: string;
  name: string;
  count: number;
}

interface PositionData {
  name: string;
  count: number;
}

interface PositionStats {
  total: number;
  distribution: PositionData[];
  topPosition: PositionData | null;
  vacantPositions: number;
}

export default function Dashboard(): React.ReactElement {
  // 获取员工数据
  const { data: staffs, isLoading: staffLoading } = api.staff.findMany.useQuery(
    {}
  );
  // 获取部门数据
  const { data: departments, isLoading: deptLoading } =
    api.department.findMany.useQuery({});

  // 部门人员分布
  const [deptDistribution, setDeptDistribution] = useState<DeptData[]>([]);
  // 选中的部门筛选
  const [selectedDept, setSelectedDept] = useState<string | null>(null);
  // 图表视图类型
  const [chartView, setChartView] = useState<string>("all");
  // 岗位统计状态
  const [positionStats, setPositionStats] = useState<PositionStats>({
    total: 0,
    distribution: [],
    topPosition: null,
    vacantPositions: 0,
  });

  // 处理原始数据，提取部门和岗位分布信息
  useEffect(() => {
    if (staffs && departments) {
      // 计算部门分布
      const deptMap = new Map<string, DeptData>();
      if (Array.isArray(departments)) {
        departments.forEach((dept) => {
          deptMap.set(dept.id, { name: dept.name, count: 0, id: dept.id });
        });
      }

      staffs.forEach((staff) => {
        if (staff.deptId && deptMap.has(staff.deptId)) {
          const deptData = deptMap.get(staff.deptId);
          if (deptData) {
            deptData.count += 1;
            deptMap.set(staff.deptId, deptData);
          }
        }
      });

      // 部门数据排序
      const deptArray = Array.from(deptMap.values())
        .filter((dept) => dept.count > 0)
        .sort((a, b) => b.count - a.count);
      setDeptDistribution(deptArray);

      // 岗位分布统计
      const positionMap = new Map<string, PositionData>();
      staffs.forEach((staff) => {
        const position = staff.positionId || "未设置岗位";
        if (!positionMap.has(position)) {
          positionMap.set(position, { name: position, count: 0 });
        }
        const posData = positionMap.get(position);
        if (posData) {
          posData.count += 1;
          positionMap.set(position, posData);
        }
      });
      // 转换为数组并排序
      const positionArray = Array.from(positionMap.values()).sort(
        (a, b) => b.count - a.count
      );
      // 找出人数最多的岗位
      const topPosition = positionArray.length > 0 ? positionArray[0] : null;
      // 计算空缺岗位数(简化示例，实际可能需要根据业务逻辑调整)
      const vacantPositions = positionArray.filter((p) => p.count === 0).length;
      // 更新岗位统计状态
      setPositionStats({
        total: positionMap.size,
        distribution: positionArray,
        topPosition,
        vacantPositions,
      });
    }
  }, [staffs, departments]);

  // 过滤部门数据
  const filteredDeptData = useMemo(() => {
    if (selectedDept) {
      return deptDistribution.filter((dept) => dept.id === selectedDept);
    }
    return deptDistribution;
  }, [deptDistribution, selectedDept]);

  // 图表配置
  const pieOptions = useMemo(() => {
    return getPieChartOptions(filteredDeptData, {});
  }, [filteredDeptData]);

  const barOptions = useMemo(() => {
    return getBarChartOptions(filteredDeptData);
  }, [filteredDeptData]);

  // 处理部门选择
  const handleDeptSelection = (params: any) => {
    const selectedDeptName = params.name;
    const selectedDept = deptDistribution.find(
      (dept) => dept.name === selectedDeptName
    );
    if (selectedDept) {
      setSelectedDept(selectedDept.id);
      message.info(`已选择部门: ${selectedDeptName}`);
    }
  };

  // 导出图表为图片
  const handleExportChart = (chartType: string, chartRef: RefObject<any>) => {
    if (chartRef.current && chartRef.current.getEchartsInstance) {
      const chart = chartRef.current.getEchartsInstance();
      const dataURL = chart.getDataURL({
        type: "png",
        pixelRatio: 2,
        backgroundColor: "#fff",
      });

      const link = document.createElement("a");
      link.download = `部门人员分布-${
        chartType === "pie" ? "饼图" : "条形图"
      }.png`;
      link.href = dataURL;
      document.body.appendChild(link);
      link.click();
      document.body.removeChild(link);

      message.success(`${chartType === "pie" ? "饼图" : "条形图"}导出成功`);
    }
  };

  // 重置筛选
  const resetFilter = () => {
    setSelectedDept(null);
    message.success("已重置筛选");
  };

  const isLoading = staffLoading || deptLoading;

  return (
    <div className="p-4">
      <h1 className="text-2xl font-bold mb-6">数据看板</h1>
      {isLoading ? (
        <div className="flex justify-center items-center h-64">
          <Spin size="large" />
        </div>
      ) : (
        <>
          {/* 统计卡片 */}
          <StatisticCards
            staffs={staffs}
            departments={departments}
            positionStats={positionStats}
          />

          {/* 图表筛选和控制区 */}
          <ChartControls
            chartView={chartView}
            setChartView={setChartView}
            selectedDept={selectedDept}
            resetFilter={resetFilter}
            deptDistribution={deptDistribution}
          />

          {/* 部门分布数据可视化 */}
          <DepartmentCharts
            chartView={chartView}
            filteredDeptData={filteredDeptData}
            pieOptions={pieOptions}
            barOptions={barOptions}
            handleExportChart={handleExportChart}
            handleDeptSelection={handleDeptSelection}
          />

          {/* 详细数据表格 */}
          <DepartmentTable
            staffs={staffs}
            filteredDeptData={filteredDeptData}
          />
        </>
      )}
    </div>
  );
}
