import { Button, Form, Input, Select, DatePicker, Radio, message, Modal, Cascader, InputNumber } from "antd";
import { useState, useMemo, useEffect } from "react";
import { api, useStaff } from "@nice/client";
import { areaOptions } from './area-options';
import InfoCard from './infoCard';
import DepartmentSelect from "@web/src/components/models/department/department-select";
import { baseFields } from "@web/src/app/admin/staffinfo-manage/defaultFields";
import dayjs from 'dayjs'; // 导入 dayjs

interface StaffInformationProps {
  staffId?: string;  // 可选，编辑模式时提供
  onComplete?: () => void;  // 可选，完成时的回调函数
  initialData?: any;  // 可选，编辑模式时提供的初始数据
  fieldValues?: any[];  // 可选，编辑模式时提供的字段值数组
  setIsEditModalVisible?: (visible: boolean) => void; // 可选，编辑模式时提供的回调函数
}
const StaffInfoWrite = ({ staffId, onComplete, initialData, fieldValues, setIsEditModalVisible }: StaffInformationProps) => {
  const [form] = Form.useForm();
  const [loading, setLoading] = useState(false);
  // 修改使用的hook方法
  const { create, update, setCustomFieldValue, useCustomFields } = useStaff();
  const { data: fields, isLoading: fieldsLoading } = useCustomFields();
  const [infoCards, setInfoCards] = useState<any[]>([]);

  // 添加跟踪培训和鉴定状态的state
  const [hasTrain, setHasTrain] = useState(false);
  const [hasCert, setHasCert] = useState(false);

  // 添加状态来跟踪每个文本区域的高度
  const [textAreaHeights, setTextAreaHeights] = useState<Record<string, number>>({});
  const { data: staffData } = api.staff.findUnique.useQuery(
    { where: { id: staffId } },
    { enabled: !!staffId }  // 只在 staffId 存在时执行查询
  );
  const handleAdd = (content: string[]) => {
    // 将数组内容展开为独立对象
    const newItems = content.map(text => ({ content: text }));
    setInfoCards(prev => [
      ...prev.filter(item => !content.includes(item.content)),
      ...newItems
    ]);
  }

  // 在组件挂载或依赖项变化时填充表单数据
  useEffect(() => {
    if (initialData && fieldValues && fields) {
      // 设置基础字段
      const formValues: any = {
        showname: initialData.showname,
        department: initialData.deptId
      };
      
      // 设置自定义字段值
      if (Array.isArray(fieldValues)) {
        fieldValues.forEach(fv => {
          // 添加类型断言确保 fields 是数组
          const field = Array.isArray(fields) ? 
            fields.find((f: any) => f.id === fv.fieldId) : undefined;
          if (field) {
            let value = fv.value;
            
            // 根据字段类型转换值的格式
            if (field.type === 'cascader' && value) {
              value = value.split('/');
            } else if (field.type === 'textarea' && value) {
              value = value.split(',');
            } else if (field.type === 'date' && value) {
              // 使用 dayjs 而不是 new Date 来处理日期
              value = value ? dayjs(value) : null;
            }
            
            formValues[field.name] = value;
            
            // 设置培训和鉴定状态
            if (field.name === 'hasTrain') {
              setHasTrain(value === '是');
            }
            if (field.name === 'hasCert') {
              setHasCert(value === '是');
            }
          }
        });
      }
      
      // 设置表单的值
      form.setFieldsValue(formValues);
    }
  }, [initialData, fieldValues, fields, form]);

  // 按分组组织字段
  const fieldGroups = useMemo(() => {
    if (!fields) return {};
    return (fields as any[]).reduce((groups: any, field: any) => {
      const group = field.group || '其他信息';
      if (!groups[group]) {
        groups[group] = [];
      }
      groups[group].push(field);
      return groups;
    }, {});
  }, [fields]);

  // 检查字段是否应该被禁用的辅助函数
  const shouldFieldBeDisabled = (field: any, groupName: string) => {
    if (groupName === '培训信息' && !hasTrain) {
      return ['trainType', 'trainInstitute', 'trainMajor'].includes(field.name);
    }
    if (groupName === '鉴定信息' && !hasCert) {
      return ['certRank', 'certWork'].includes(field.name);
    }
    return false;
  };

  const renderField = (field: any, groupName: string) => {
    // 处理培训和鉴定的单选按钮
    if (field.name === 'hasTrain') {
      return (
        <Radio.Group
          options={[
            { label: '是', value:'是' },
            { label: '否', value:'否' }
          ]}
          onChange={(e) => {
            setHasTrain(e.target.value);
            form.setFieldsValue({ hasTrain: e.target.value });
          }}
          value={hasTrain}
        />
      );
    }

    if (field.name === 'hasCert') {
      return (
        <Radio.Group
          options={[
            { label: '是', value: '是' },
            { label: '否', value: '否' }
          ]}
          onChange={(e) => {
            setHasCert(e.target.value);
            form.setFieldsValue({ hasCert: e.target.value });
          }}
          value={hasCert}
        />
      );
    }

    // 检查字段是否应禁用
    const isDisabled = shouldFieldBeDisabled(field, groupName);


    // 根据字段类型渲染不同的组件
    switch (field.type) {
      case 'text':
        if(field.name === 'department'){
          return <DepartmentSelect disabled={isDisabled} />;
        }else{
          return <Input disabled={isDisabled} />;
        }
      case 'number':
        return <InputNumber disabled={isDisabled} />;
      case 'date':
        return <DatePicker disabled={isDisabled} />;
      case 'select':
        if (field.options && field.options.length > 0) {
          return <Select options={field.options} disabled={isDisabled} />;
        }
        return <Input placeholder="选项数据缺失" disabled={isDisabled} />;
      case 'radio':
        if (field.options && field.options.length > 0) {
          return <Radio.Group options={field.options} disabled={isDisabled} />;
        }
        return <Input placeholder="选项数据缺失" disabled={isDisabled} />;
      case 'textarea':
        return (
          <div
            className="w-full relative"
            style={{
              minHeight: textAreaHeights[field.name] ?
                `${textAreaHeights[field.name]}px` : 'auto'
            }}
          >
            <InfoCard 
              onAdd={(contents) => {
                handleAdd(contents);
                // 同步更新表单字段值
                form.setFieldValue(field.name, contents);
              }}
              onHeightChange={(height) => {
                setTextAreaHeights(prev => ({
                  ...prev,
                  [field.name]: height
                }));
              }}
            />
          </div>
        );
      case 'cascader':
        if (field.label === '籍贯') {
          return <Cascader options={areaOptions} disabled={isDisabled} />;
        } else {
          return <Input disabled={isDisabled} />;
        }
      default:
        return <Input className="w-full" disabled={isDisabled} />;
    }
  };
  const onFinish = async (e, values: any) => {
    e.preventDefault()
    console.log(values)
    try {
      setLoading(true);
      // 创建基础员工记录
      if (!values.showname) {
        message.error("姓名不能为空");
        return;
      }
      
      let staff;
      
      if (staffId) {
        // 编辑模式 - 更新现有员工
        console.log('准备更新用户，数据:', { id: staffId, showname: values.showname });
        try {
          staff = await update.mutateAsync({
            where: { id: staffId },
            data: {
              showname: values.showname,
              deptId: values.department || null,
            }
          });
          console.log('更新员工记录:', staff);
        } catch (error) {
          console.error('更新员工基本信息失败:', error);
          message.error("更新失败，请重试");
          setLoading(false);
          return; // 如果基本信息更新失败，提前返回不继续处理
        }
      } else {
        // 创建模式 - 创建新员工
        console.log('准备创建用户，数据:', { showname: values.showname });
        try {
          staff = await create.mutateAsync({
            data: {
              showname: values.showname,
              deptId: values.department ? values.department : null,
            }
          });
        } catch (error) {
          console.error('创建员工记录失败:', error);
          message.error("提交失败，请重试");
          setLoading(false);
          return; // 如果创建失败，提前返回不继续处理
        }
      }
      
      // 只有在上面的操作成功后才继续处理自定义字段
      try {
        // 过滤有效字段并转换值
        const validEntries = Object.entries(values)
          .filter(([key, value]) => 
            key !== 'showname' && 
            key !== 'username' && 
            key !== 'department' &&
            value !== undefined && 
            value !== null && 
            value !== ''
          )
          .map(([fieldName, value]) => {
            const field = fields && Array.isArray(fields) ? 
              fields.find((f: any) => f.name === fieldName) : undefined;
            
            // 处理特殊字段类型
            let processedValue = value;
            if (field?.type === 'date') {
              processedValue = (value as any)?.format?.('YYYY-MM-DD') || value;
            } else if (field?.type === 'cascader' && Array.isArray(value)) {
              processedValue = value?.join('/');
            } else if(field?.type === 'textarea'){
              processedValue = Array.isArray(value) ? value.join(',') : value;
            }

            return { field, value: processedValue };
          })
          .filter(item => item.field?.id);
    
        // 批量提交自定义字段
        await Promise.all(
          validEntries.map(({ field, value }) =>
            setCustomFieldValue.mutateAsync({
              staffId: staff.id,
              fieldId: field.id,
              value: String(value)
            })
          )
        );
        
        // 只有当所有操作都成功时才显示成功消息
        message.success(staffId ? "信息更新成功" : "信息提交成功");
        
        // 关闭编辑模态窗口（如果在编辑模式）
        if (staffId && setIsEditModalVisible) {
          setIsEditModalVisible(false);
        }
        
        // 如果有回调函数，调用它
        if (onComplete) {
          onComplete();
        } else if (!staffId) {
          // 如果是新建模式，重置表单
          form.resetFields();
        }
      } catch (error) {
        console.error('设置自定义字段值失败:', error);
      }
    } catch (error) {
      console.error('提交出错:', error);
      message.error(staffId ? "更新失败，请重试" : "提交失败，请重试");
    } finally {
      setLoading(false);
    }
  };

  if (fieldsLoading) {
    return <div>加载中...</div>;
  }

  return (
    <div className="max-w-4xl mx-auto p-6">
      <h1 className="text-2xl font-bold mb-6">人员信息填报</h1>

      <Form
        form={form}
        layout="vertical"
        onValuesChange={(changedValues) => {
          if ('hasTrain' in changedValues) {
            setHasTrain(!!changedValues.hasTrain);
          }
          if ('hasCert' in changedValues) {
            setHasCert(!!changedValues.hasCert);
          }
        }}
        className="space-y-6 mt-6"
      >
        {/* 新增基础信息区块 */}
      <div className="bg-blue-45 p-6 rounded-lg shadow relative mb-6">
        
        <div className="grid grid-cols-1 md:grid-cols-2 gap-x-6 gap-y-6">
          {baseFields.map((field) => (
            <Form.Item
              key={field.name}
              label={field.label}
              name={field.name}
              rules={[
                {
                  required: field.required,
                  message: `请输入${field.label}`,
                },
              ]}
            >
              {renderField(field, '系统默认信息')}
            </Form.Item>
          ))}
        </div>
      </div>
        {Object.entries(fieldGroups).map(([groupName, groupFields]) => (
          <div key={groupName} className="bg-white p-6 rounded-lg shadow relative">
            <h2 className="text-lg font-semibold mb-4">{groupName}</h2>
            <div className={`
              grid grid-cols-1 md:grid-cols-2 gap-x-6 gap-y-12
              ${groupName === "工作信息" ? "md:[&>*:nth-child(odd)]:col-start-1 md:[&>*:nth-child(even)]:col-start-2" : ""}
            `}>
              {(groupFields as any[])?.map((field: any) => (
                <Form.Item
                  key={field.id}
                  label={<span className="block mb-2">{field.label}</span>}
                  name={field.name}
                  rules={[
                    {
                      required: field.required,
                      message: `请输入${field.label}`,
                    },
                  ]}
                  className={`
                    ${field.type === 'textarea' && groupName !== "工作信息" ? "md:col-span-2" : ""}
                    ${field.type === 'textarea' ? "transition-all duration-200 ease-in-out" : ""} 
                  `}
                  style={{
                    minHeight: field.type === 'textarea' && textAreaHeights[field.name] ?
                      `${textAreaHeights[field.name] + 50}px` : 'auto'
                  }}
                >
                  {renderField(field, groupName)}
                </Form.Item>
              ))}
            </div>
          </div>
        ))}

        <div className="flex justify-end space-x-4">
          <Button onClick={() => form.resetFields()}>重置</Button>
          <Button
            type="primary"
            htmlType="submit"
            onClick={(e) => onFinish(e, form.getFieldsValue())}
            loading={loading}
          >
            提交
          </Button>
        </div>
      </Form>
    </div>
  );
};

export default StaffInfoWrite;